@@ -22,11 +22,8 @@
 #ifndef ACTIONS_H
 #define ACTIONS_H
 
-#include <gtk/gtk.h>
+#include <gthumb.h>
 
-#define DEFINE_ACTION(x) void x (GtkAction *action, gpointer data);
-
-DEFINE_ACTION(gth_browser_activate_action_export_23)
-DEFINE_ACTION(gth_browser_activate_action_import_23)
+DEF_ACTION_CALLBACK (gth_browser_activate_export_23)
 
 #endif /* ACTIONS_H */
