@@ -25,8 +25,7 @@
 #include <gthumb.h>
 
 void      edit_metadata__gth_browser_construct_cb              (GthBrowser  *browser);
-void      edit_metadata__gth_browser_update_sensitivity_cb     (GthBrowser  *browser);
-gpointer  edit_metadata__gth_browser_file_list_key_press_cb    (GthBrowser  *browser,
-						   	        GdkEventKey *event);
+void      edit_metadata__gth_browser_selection_changed_cb      (GthBrowser  *browser,
+							        int          n_selected);
 
 #endif /* CALLBACKS_H */
