@@ -30,6 +30,7 @@
 #include "gth-browser-actions-callbacks.h"
 #include "gth-file-list.h"
 #include "gth-file-selection.h"
+#include "gth-filterbar.h"
 #include "gth-folder-tree.h"
 #include "gth-main.h"
 #include "gth-preferences.h"
@@ -425,6 +426,15 @@ gth_browser_activate_open_folder_in_new_window (GSimpleAction *action,
 }
 
 
+void
+gth_browser_activate_show_progress_dialog (GSimpleAction *action,
+					   GVariant      *parameter,
+					   gpointer       user_data)
+{
+	gth_browser_show_progress_dialog (GTH_BROWSER (user_data));
+}
+
+
 void
 gth_browser_activate_show_hidden_files (GSimpleAction *action,
 					GVariant      *state,
@@ -616,3 +626,31 @@ gth_browser_activate_show_menu (GSimpleAction *action,
 {
 	gth_browser_show_menu (GTH_BROWSER (user_data));
 }
+
+
+void
+gth_browser_activate_set_filter (GSimpleAction *action,
+				 GVariant	*parameter,
+				 gpointer	 user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	const char *test_id;
+	GtkWidget  *filterbar;
+
+	test_id = g_variant_get_string (parameter, NULL);
+	filterbar = gth_browser_get_filterbar (browser);
+	gth_filterbar_set_test_by_id (GTH_FILTERBAR (filterbar), test_id);
+}
+
+
+void
+gth_browser_activate_set_filter_all (GSimpleAction *action,
+				     GVariant      *parameter,
+				     gpointer        user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	GtkWidget  *filterbar;
+
+	filterbar = gth_browser_get_filterbar (browser);
+	gth_filterbar_set_show_all (GTH_FILTERBAR (filterbar));
+}
