@@ -54,6 +54,7 @@ DEF_ACTION_CALLBACK (gth_browser_activate_viewer_edit_file)
 DEF_ACTION_CALLBACK (gth_browser_activate_viewer_properties)
 DEF_ACTION_CALLBACK (gth_browser_activate_unfullscreen)
 DEF_ACTION_CALLBACK (gth_browser_activate_open_folder_in_new_window)
+DEF_ACTION_CALLBACK (gth_browser_activate_show_progress_dialog)
 DEF_ACTION_CALLBACK (gth_browser_activate_show_hidden_files)
 DEF_ACTION_CALLBACK (gth_browser_activate_sort_by)
 DEF_ACTION_CALLBACK (gth_browser_activate_show_statusbar)
@@ -70,5 +71,7 @@ DEF_ACTION_CALLBACK (gth_browser_activate_apply_editor_changes)
 DEF_ACTION_CALLBACK (gth_browser_activate_select_all)
 DEF_ACTION_CALLBACK (gth_browser_activate_unselect_all)
 DEF_ACTION_CALLBACK (gth_browser_activate_show_menu)
+DEF_ACTION_CALLBACK (gth_browser_activate_set_filter)
+DEF_ACTION_CALLBACK (gth_browser_activate_set_filter_all)
 
 #endif /* GTH_BROWSER_ACTIONS_CALLBACK_H */
